# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

sys.path.insert(0, os.path.abspath('../..'))
# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinxcontrib.apidoc',
    'sphinx.ext.intersphinx',
    'openstackdocstheme',
    'oslo_config.sphinxext',
]

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/oslo.metrics'
openstackdocs_bug_project = 'oslo.metrics'
openstackdocs_bug_tag = ''

# sphinxcontrib.apidoc options
apidoc_module_dir = '../../oslo_metrics'
apidoc_output_dir = 'reference/api'
apidoc_excluded_paths = ['tests']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'oslo.metrics'
copyright = '2014, OpenStack Foundation'

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# A list of ignored prefixes for module index sorting.
modindex_common_prefix = ['oslo_metrics.']

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
html_theme = 'openstackdocs'
# html_static_path = ['static']

# Output file base name for HTML help builder.
htmlhelp_basename = f'{project}doc'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    (
        'index',
        f'{project}.tex',
        f'{project} Documentation',
        'OpenStack Foundation',
        'manual',
    )
]

intersphinx_mapping = {
    'python': ('https://docs.python.org/', None),
    'osloconfig': ('http://docs.openstack.org/oslo.config/latest/', None),
}
